from django.db import models
from django.template.loader import render_to_string

from .models import Article


class ArticleList(models.Model):
    number = models.IntegerField()

    class Meta:
        abstract = True

    def get_queryset_for_render(self):
        return Article.objects.all()

    def render(self, **kwargs):
        context = {
            'object_list': self.get_queryset_for_render()[:self.number],
            'request': kwargs.get('request'),
        }
        return render_to_string('content/articles/list.html', context)



from django import forms
from django.db import models
from django.contrib.admin.widgets import AdminRadioSelect
from django.template.loader import render_to_string
from django.utils.translation import ugettext_lazy as _
from feincms.admin.item_editor import ItemEditorForm

from articles.models import Article


class ArticleCategoryList(models.Model):
    """
    List articles belonging to the selected category.
    """
    category = models.ForeignKey('articles.Category')
    number = models.IntegerField()

    class Meta:
        abstract = True
        verbose_name = _('article category list')

    @classmethod
    def initialize_type(cls, LAYOUT_CHOICES=None):

        cls.add_to_class('layout', models.CharField(_('Layout'),
                                                    max_length=10, choices=LAYOUT_CHOICES,
                                                    default=LAYOUT_CHOICES[0][0]))

        class ArticleCategoryListForm(ItemEditorForm):
            layout = forms.ChoiceField(choices=LAYOUT_CHOICES,
                                       initial=LAYOUT_CHOICES[0][0], label=_('Layout'),
                                       widget=AdminRadioSelect(attrs={'class': 'radiolist'}))

        cls.feincms_item_editor_form = ArticleCategoryListForm
        cls.form = ArticleCategoryListForm

    def get_queryset_for_render(self):
        return Article.objects.filter(category=self.category)

    def render(self, **kwargs):
        context = {
            'object_list': self.get_queryset_for_render()[:self.number],
            'request': kwargs.get('request'),
            'content': self,
        }
        return render_to_string([
            'content/articles/category/%s/%s.html' % (self.region, self.layout),
            'content/articles/category/%s/default.html' % self.region,
            'content/articles/category/%s.html' % self.layout,
            'content/articles/category/default.html',
            ], context)


class ArticleList(models.Model):
    """
    List articles belonging to any of the selected categories.
    If no categories are selected then list all articles.
    """
    number = models.IntegerField()
    categories = models.ManyToManyField('articles.Category', null=True, blank=True)

    class Meta:
        abstract = True
        verbose_name = _('article list')

    def get_queryset_for_render(self):
        articles = Article.objects.all()
        if self.categories.count():
            articles = articles.filter(category__in=self.categories.all())
        return articles

    def render(self, **kwargs):
        context = {
            'object_list': self.get_queryset_for_render()[:self.number],
            'request': kwargs.get('request'),
            'content': self,
        }
        return render_to_string(['content/articles/%s/list.html' % self.region,
                                 'content/articles/list.html',
                                ],
                                context)



from django import template

from ..models import Article
from ..utils import parse_tokens

register = template.Library()


class ArticlesNode(template.Node):
    """
        Output a list of articles.
        If as varname is specified then add the result to the context.

        Usage:
            {% articles %}
            OR
            {% articles articles %}
            OR
            {% articles articles limit %}
            OR
            {% articles as artilce_list %}
            OR
            {% articles articles as artilce_list %}
            OR
            {% articles limit=limit as artilce_list %}
    """
    def __init__(self, articles=None, limit=None, varname=None):
        self.articles = articles
        self.limit = limit
        self.varname = varname

    def render(self, context):
        articles = self.articles and self.articles.resolve(context)
        limit = self.limit and self.limit.resolve(context)


        if articles is None:
            user = None
            if 'request' in context:
                user = context['request'].user
            articles = Article.objects.active(user=user).select_related()

        if limit is not None:
            articles = articles[:limit]


        if self.varname is not None:
            context[self.varname] = articles
            return ''
        else:
            t = template.loader.select_template(['articles/articles.html'])
            context.push()
            context['articles'] = articles
            output = t.render(context)
            context.pop()

            return output


@register.tag()
def articles(parser, token):
    bits = token.split_contents() 

    varname = None
    try:
        if bits[-2] == 'as':
            varname = bits[-1]
            bits = bits[:-2]
    except IndexError:
        pass

    args, kwargs = parse_tokens(parser, bits)
    if varname is not None:
        kwargs['varname'] = varname

    return ArticlesNode(*args, **kwargs)




from django.views.generic import DetailView, ListView

from .models import Article


class AppContentMixin(object):
    def render_to_response(self, context, **response_kwargs):
        """
        Returns the template tuple needed for FeinCMS App Content.
        """
        if hasattr(self.request, '_feincms_extra_context') and 'app_config' in self.request._feincms_extra_context:
            return (self.get_template_names(), context)

        return super(AppContentMixin, self).render_to_response(context, **response_kwargs)


class ArticleDetail(AppContentMixin, DetailView):
    model = Article

    def get_queryset(self):
        return Article.objects.active()


class ArticleList(AppContentMixin, ListView):
    model = Article

    def get_queryset(self):
        return Article.objects.active()



from django import template

from articles.modules.category.models import Category
from articles.utils import parse_tokens

register = template.Library()

class CategoriesNode(template.Node):
    """
        Output a list of categories.

        Usage:
            {% categories %}
    """
    def __init__(self, selected=None, current=None):
        self.selected = selected
        self.current = current

    def render(self, context):
        selected = self.selected and self.selected.resolve(context)
        current = self.current and self.current.resolve(context)

        user = 'request' in context and context['request'].user or None
        categories = None
        if current is None:
            categories = Category.objects.active(user=user).filter(parent__isnull=True)
        else:
            if selected is not None:
                # is the selected category a descendant of 
                if current.get_descendants(include_self=True).filter(pk=selected.pk).count() > 0:
                    categories = current.children.filter(Category.objects.active_query(user=user))

        if categories is not None:
            categories = categories.distinct()

        t = template.loader.select_template(['articles/categories.html'])
        context.push()
        context['selected'] = selected
        context['categories'] = categories
        output = t.render(context)
        context.pop()

        return output


@register.tag()
def articlecategories(parser, token):
    bits = token.split_contents() 

    args, kwargs = parse_tokens(parser, bits)

    return CategoriesNode(*args, **kwargs)




from django.db.models.fields import FieldDoesNotExist
from haystack import indexes

from models import Article


class TempArticleIndex(indexes.SearchIndex):
    title = indexes.CharField(model_attr='title')
    name = indexes.CharField(model_attr='title')
    text = indexes.CharField(document=True, use_template=True)

    def get_model(self):
        return Article

    def index_queryset(self):
        return self.get_model().objects.active()

    def get_updated_field(self, **kwargs):
        try:
            self.get_model()._meta.get_field('modification_date')
        except FieldDoesNotExist:
            return None
        else:
            return 'modification_date'


try:
    # In haystack < 2.0 we need to explicitly register indexes
    from haystack import site
except ImportError:
    # In haystack >= 2.0 Indexes subclass indexes.Indexable
    class ArticleIndex(TempArticleIndex, indexes.Indexable):
        pass
else:
    class ArticleIndex(TempArticleIndex):
        def get_queryset(self):
            return self.index_queryset()

    site.register(Article, ArticleIndex)



import datetime
import warnings

from django.core.urlresolvers import reverse
from django.test import TestCase

from .models import Article


def find(f, seq):
    """
    Return first item in sequence where f(item) == True.
    e.g.
    fred = find(lambda person: person.name == 'Fred', peeps)
    """
    for item in seq:
        if f(item):
            return item


class ArticleAccessTests(TestCase):
    fixtures = ['articles_data.json',]
    def test_article_index(self):
        response = self.client.get(reverse('article_index'))
        for article in Article.objects.active():
            self.assertContains(response, article.title)

    def test_article_detail(self):
        response = self.client.get(reverse('article_detail', args=['test-article',]))


        article = Article.objects.active().get(slug='test-article')
        self.assertContains(response, article.title)

class ArticleActiveTests(TestCase):
    fixtures = ['articles_data.json',]
    def test_article_active(self):
        response = self.client.get(reverse('article_index'))


        inactive_articles = Article.objects.exclude(pk__in=[a[0] for a in Article.objects.active().values_list('pk')])
        assert(inactive_articles)
        for article in inactive_articles:
            self.assertNotContains(response, article.title)

    def test_article_views_404(self):
        response = self.client.get(reverse('article_detail', args=['inactive-article',]))
        self.assertEquals(response.status_code, 404)

# extension related tests
class ArticleDatePublisherTests(TestCase):
    fixtures = ['articles_datepublisher_data.json',]

    def setUp(self, *args, **kwargs):
        if bool(find(lambda f: f.name == 'publication_date', Article._meta.local_fields)) \
           and bool(find(lambda f: f.name == 'publication_end_date', Article._meta.local_fields)):
            self.skip = False
        else: 
            warnings.warn("Skipping datepublisher tests. Extension not registered")
            self.skip = True

    def test_publication_date(self):
        if self.skip:
            return

        article = Article.objects.active().get(slug='publication-date-test')
        article.publication_date = datetime.datetime.now() + datetime.timedelta(1)
        article.save()

        response = self.client.get(reverse('article_detail', args=['publication-date-test',]))
        self.assertEquals(response.status_code, 404)


        article.publication_date = datetime.datetime.now() + datetime.timedelta(-1)
        article.publication_end_date = datetime.datetime.now() + datetime.timedelta(-1)
        article.save()

        response = self.client.get(reverse('article_detail', args=['publication-date-test',]))
        self.assertEquals(response.status_code, 404)


class ArticleTagsTests(TestCase):
    fixtures = ['articles_tags_data.json',]

    def setUp(self, *args, **kwargs):
        if bool(find(lambda f: f.name == 'tags', Article._meta.many_to_many)):
            self.skip = False
        else: 
            warnings.warn("Skipping tags tests. Extension not registered")
            self.skip = True

    def test_tags(self):
        if self.skip:
            return

        article = Article.objects.active().get(slug='tag-test')
        article.tags.add("test", "testing")

        response = self.client.get(reverse('article_tagged_list', args=['test',]))
        self.assertContains(response, article.title)

    def test_tags_404(self):
        response = self.client.get(reverse('article_tagged_list', args=['tag_does_not_exist',]))
        self.assertEquals(response.status_code, 404)




from django.db import models
from django.utils.translation import ugettext_lazy as _
from feincms import extensions


class Extension(extensions.Extension):
    def handle_model(self):
        self.model.add_to_class(
            'thumbnail',
            models.ImageField(
                _('thumbnail'),
                max_length=250,
                upload_to="images/articles/thumbnails",
                null=True,
                blank=True)
        )

    def handle_modeladmin(self, modeladmin):
        modeladmin.add_extension_options(_('Thumbnail'), {
            'fields': ('thumbnail',),
        })



import warnings

from django.contrib.gis import admin
from django.contrib.gis.db import models
from django.utils.translation import ugettext_lazy as _
from feincms import extensions


class Extension(extensions.Extension):
    def handle_model(self):
        self.model.add_to_class(
            'location',
            models.PointField(verbose_name=_('location'), null=True, blank=True))

        from articles.models import ArticleManager

        class GeoArticleManager(ArticleManager, models.GeoManager):
            pass

        self.model.add_to_class('objects', GeoArticleManager())

    def handle_modeladmin(self, modeladmin):
        if not issubclass(modeladmin, admin.OSMGeoAdmin):
            warnings.warn(
                "The admin class articles ArticleAdmin class is not a sub class of django.contrib.gis.admin.OSMGeoAdmin. "
                "Consider setting ARTICLE_MODELADMIN_CLASS = 'django.contrib.gis.admin.OSMGeoAdmin'")

        modeladmin.add_extension_options(_('Location'), {
            'fields': ('location',),
            'classes': ('collapse',),
        })



from django.conf.urls import patterns, url


urlpatterns = patterns('articles.modules.category.views',
    url(r'^(?P<category_url>[a-z0-9_/-]+/)(?P<article>[a-z0-9_-]+).html$', 'article_detail', name="article_detail"),
    url(r'^(?P<category_url>[a-z0-9_/-]+/)$', 'article_category', name='article_category'),
    url(r'^$', 'article_category', name='article_index'),
)





from django import template


def parse_tokens(parser, bits):
    """
    Parse a tag bits (split tokens) and return a list on kwargs (from bits of the  fu=bar) and a list of arguments.
    """

    kwargs = {}
    args = []
    for bit in bits[1:]:
        try:
            try:
                pair = bit.split('=')
                kwargs[str(pair[0])] = parser.compile_filter(pair[1])
            except IndexError:
                args.append(parser.compile_filter(bit))
        except TypeError:
            raise template.TemplateSyntaxError('Bad argument "%s" for tag "%s"' % (bit, bits[0]))

    return args, kwargs



from django.conf.urls import patterns, url
from django.core.exceptions import ImproperlyConfigured
from django.utils.translation import ugettext_lazy as _
from feincms import extensions

try:
    from taggit.managers import TaggableManager
except ImportError:
    raise ImproperlyConfigured('You need to install django-taggit to use the tags extension')


class Extension(extensions.Extension):
    def handle_model(self):
        self.model.add_to_class('tags', TaggableManager(verbose_name=_('tags'), blank=True))
        self.model.get_urlpatterns_orig = self.model.get_urlpatterns

        @classmethod
        def get_urlpatterns(cls):
            taggit_patterns = patterns('taggit.views',
                url(r'^tags/(?P<slug>[^/]+)/$', 'tagged_object_list', {
                    'queryset': cls.objects.active}, name="article_tagged_list"
                ),
            )
            return cls.get_urlpatterns_orig() + taggit_patterns
        self.model.get_urlpatterns = get_urlpatterns

    def handle_modeladmin(self, modeladmin):
        modeladmin.add_extension_options(_('Tags'), {
            'fields': ('tags',),
        })



from django.conf.urls import patterns, url
from django.db import models
from django.utils.translation import ugettext_lazy as _
from feincms.content.application import models as app_models

from feincms import extensions


class Extension(extensions.Extension):

    def handle_model(self):
        self.model.add_to_class('category', models.ForeignKey('articles.Category', verbose_name=_('category')))
        self.model._meta.unique_together += [('category', 'slug')]
        self.model.get_urlpatterns_orig = self.model.get_urlpatterns

        @classmethod
        def get_urlpatterns(cls):
            from articles.modules.category import views
            return patterns('',
                    url(r'^(?P<category_url>[a-z0-9_/-]+/)articles/(?P<slug>[a-z0-9_-]+)/$', views.CategoryArticleDetail.as_view(), name="article_detail"),
                    url(r'^(?P<category_url>[a-z0-9_/-]+/)articles/$', views.CategoryArticleList.as_view(), name='article_category'),
                    url(r'^$', views.CategoryArticleList.as_view(), name='article_index'),
            )
        self.model.get_urlpatterns = get_urlpatterns

        def get_absolute_url(self):
            return ('article_detail', 'articles.urls', (), {
                    'category_url': self.category.local_url,
                    'slug': self.slug,
                    })
        self.model.get_absolute_url = app_models.permalink(get_absolute_url)

    def handle_modeladmin(self, modeladmin):
        modeladmin.list_filter += ['category', ]
        modeladmin.list_display.insert(1, 'category', )
        modeladmin.add_extension_options(_('Category'), {
            'fields': ('category',),
        })



from setuptools import setup, find_packages


setup(
    name='feincms-articles',
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'FeinCMS>=1.7',
        'django-mptt',
        'django-pagination',
    ],
    version='1.1',
    description='Provides Articles using (FeinCMS content) with categories and tags.',
    author='Incuna Ltd',
    author_email='admin@incuna.com',
    url='https://github.com/incuna/feincms-articles/',
)



from django.conf import settings
from django.db.models import Q
from django.http import HttpResponseRedirect
from django.shortcuts import get_object_or_404

from .models import Category
from articles.views import ArticleDetail, ArticleList


class CategoryAccesssGroupsMixin(object):
    def has_access_groups_permission(self, category):
        """
        Check that the current user has (access_group) permission to access the category.
        Return a (redirect) request object if the user does not have permission.
        Return none if the user has permissions
        """
        if category:
            access_groups = category.access_groups.all()
            user = self.request.user
            if access_groups and (user is None or not user.is_authenticated() or not user in access_groups):
                return False

        return True


class CategoryArticleDetail(ArticleDetail, CategoryAccesssGroupsMixin):
    template_name = "articles/category_article_detail.html"
    def get_queryset(self):
        return super(CategoryArticleDetail, self).get_queryset().filter(category__local_url=self.kwargs['category_url'])

    def get(self, request, *args, **kwargs):
        self.object = self.get_object()
        context = self.get_context_data(object=self.object)

        if not self.has_access_groups_permission(self.object.category):
            return HttpResponseRedirect("%s?next=%s" % (settings.LOGIN_URL, self.request.path))

        return self.render_to_response(context)


class CategoryArticleList(ArticleList, CategoryAccesssGroupsMixin):
    template_name = "articles/category_article_list.html"
    category = None

    def get(self, request, *args, **kwargs):

        if 'category_url' in self.kwargs:
            self.category = get_object_or_404(Category, local_url=self.kwargs['category_url'])
            if not self.has_access_groups_permission(self.category):
                return HttpResponseRedirect("%s?next=%s" % (settings.LOGIN_URL, self.request.path))

        elif getattr(settings, 'ARTICLE_SHOW_FIRST_CATEGORY', False):
            # Redirect to the first category
            try:
                return HttpResponseRedirect(Category.objects.active(user=self.request.user)[0].get_absolute_url())
            except IndexError as e:
                pass


        return super(CategoryArticleList, self).get(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super(CategoryArticleList, self).get_context_data(**kwargs)
        context['category'] = self.category

        return context

    def get_queryset(self):

        articles = super(CategoryArticleList, self).get_queryset()
        user = self.request.user

        # Limit the articles based on the category access_group permission
        if user is not None and user.is_authenticated():
            query = Q(category__access_groups__isnull=True) | Q(category__access_groups__in=user.groups.all())
        else:
            query = Q(category__access_groups__isnull=True)
        articles = articles.filter(query)

        if self.category:
            if getattr(settings, 'ARTICLE_SHOW_DESCENDANTS', False):
                articles = articles.filter(category__in=self.category.get_descendants(include_self=True)).order_by(self.category.order_by)
            else:
                articles = articles.filter(category=self.category).order_by(self.category.order_by)

        return articles.select_related('category')



from django.db import models
from django.db.models import Q
from django.utils.translation import ugettext_lazy as _
from django.conf.urls import patterns, url
from django.utils.encoding import python_2_unicode_compatible

from feincms.content.application import models as app_models
from feincms.models import Base
from feincms.module.mixins import ContentModelMixin
from feincms.utils.managers import ActiveAwareContentManagerMixin


class ArticleManager(ActiveAwareContentManagerMixin, models.Manager):
    active_filters = {'simple-active': Q(active=True)}


@python_2_unicode_compatible
class BaseArticle(ContentModelMixin, Base):
    active = models.BooleanField(_('active'), default=True)

    title = models.CharField(_('title'), max_length=255)
    slug = models.SlugField(_('slug'), max_length=255, help_text=_('This will be automatically generated from the name'), unique=True, editable=True)

    class Meta:
        ordering = ['title']
        unique_together = []
        verbose_name = _('article')
        verbose_name_plural = _('articles')
        abstract = True

    objects = ArticleManager()

    @classmethod
    def get_urlpatterns(cls):
        import views
        return patterns('',
            url(r'^$', views.ArticleList.as_view(), name='article_index'),
            url(r'^(?P<slug>[a-z0-9_-]+)/$', views.ArticleDetail.as_view(), name='article_detail'),
        )

    @classmethod
    def remove_field(cls, f_name):
        """Remove a field. Effectively inverse of contribute_to_class"""
        # Removes the field form local fields list
        cls._meta.local_fields = [f for f in cls._meta.local_fields if f.name != f_name]

        # Removes the field setter if exists
        if hasattr(cls, f_name):
            delattr(cls, f_name)

    @classmethod
    def get_urls(cls):
        return cls.get_urlpatterns()

    def __str__(self):
        return self.title

    @app_models.permalink
    def get_absolute_url(self):
        return ('article_detail', 'articles.urls', (), {'slug': self.slug})

    @property
    def is_active(self):
        return self.__class__.objects.active().filter(pk=self.pk).count() > 0



from django.conf import settings
from django.core.urlresolvers import get_callable

try:
    from feincms.admin.item_editor import ItemEditor
except ImportError:
    from feincms.admin.editor import ItemEditor

from feincms.extensions import ExtensionModelAdmin

from articles.bases import BaseArticle


class Article(BaseArticle):
    pass


ModelAdmin = get_callable(getattr(settings, 'ARTICLE_MODELADMIN_CLASS', 'django.contrib.admin.ModelAdmin'))


class ArticleAdmin(ItemEditor, ExtensionModelAdmin):
    list_display = ['title', 'active',]
    list_filter = []
    search_fields = ['title', 'slug']
    filter_horizontal = []
    prepopulated_fields = {
        'slug': ('title',),
    }
    fieldsets = [
        (None, {
            'fields': ['active', 'title', 'slug']
        }),
        # <-- insertion point, extensions appear here, see insertion_index above
    ]

    fieldset_insertion_index = 1




import mptt
from denorm import denormalized, depend_on_related
from django.conf import settings
from django.core.urlresolvers import get_callable
from django.db import models
from django.db.models import Q
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _
from feincms.admin import tree_editor as editor
from feincms.content.application import models as app_models

from articles.models import Article


class CategoryManager(models.Manager):

    def active_query(self, user=None):

        if user is not None and user.is_authenticated():
            query = Q(access_groups__isnull=True) | Q(access_groups__in=user.groups.all())
        else:
            query = Q(access_groups__isnull=True)

        return query

    def active(self, user=None):
        """Active categories (containing active articles)"""

        return self.filter(self.active_query(user=user)).distinct()


@python_2_unicode_compatible
class Category(models.Model):
    ORDER_BY_CHOICES = (('publication_date', _('Publication date (oldest first)')),
                        ('-publication_date', _('Publication date (newest first)')),
                        ('title', _('Title A-Z')),
                        ('-title', _('Title Z-A')),
                       )

    name = models.CharField(_('name'), max_length=255)
    slug = models.SlugField(_('slug'), max_length=255, help_text=_('This will be automatically generated from the name'),unique=True,editable=True)
    parent = models.ForeignKey('self', verbose_name=_('parent'), blank=True, null=True, related_name='children')
    order_by = models.CharField(_('articles order'), max_length=30, choices=ORDER_BY_CHOICES, help_text=_('The order of article items in this category.'), default='-publication_date')

    access_groups  = models.ManyToManyField("auth.Group", verbose_name=_('access groups'), null=True, blank=True,
                                            help_text=_('Users must be logged in and a member of the group(s) to access this group.'), )

    @denormalized(models.CharField, max_length=255, editable=False, default='', db_index=True)
    @depend_on_related('self',type='forward')
    def local_url(self):
        if self.parent:
            root = self.parent.local_url
        else:
            root = ''
        return u'%s%s/' % (root, self.slug)

    @property
    def descendant_articles(self):
        return Article.objects.filter(category__in=self.get_descendants(include_self=True))

    objects = CategoryManager()

    class Meta:
        app_label = 'articles'
        ordering = ['tree_id', 'lft']
        verbose_name = _('category')
        verbose_name_plural = _('categories')

    def __str__(self):
        return self.name

    @app_models.permalink
    def get_absolute_url(self):
        return ('article_category', 'articles.urls', (self.local_url,))

mptt.register(Category)


ModelAdmin = get_callable(getattr(settings, 'CATEGORY_MODELADMIN_CLASS', 'django.contrib.admin.ModelAdmin'))


class CategoryAdmin(editor.TreeEditor, ModelAdmin):
    list_display = ['name', 'order_by']
    list_filter = ['parent',]
    prepopulated_fields = {
        'slug': ('name',),
    }
